# -*- coding: utf-8 -*-
# copyright 2013 LOGILAB S.A. (Paris, FRANCE), all rights reserved.
# contact http://www.logilab.fr -- mailto:contact@logilab.fr
#
# This program is free software: you can redistribute it and/or modify it under
# the terms of the GNU Lesser General Public License as published by the Free
# Software Foundation, either version 2.1 of the License, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
# details.
#
# You should have received a copy of the GNU Lesser General Public License along
# with this program. If not, see <http://www.gnu.org/licenses/>.

"""cubicweb-vcwiki controllers' tests"""

from os import path as osp

import hglib
from cubicweb.web import Redirect

from utils import VCWikiTC


class VCWikiViewControllerTC(VCWikiTC):

    def publish_wiki_path(self, path, wikiid='vcwiki'):
        with self.admin_access.web_request() as req:
            req.form = {'wikiid': wikiid, 'wikipath': path}
            return self.ctrl_publish(req, ctrl='wiki')

    def test_wiki_does_not_exist(self):
        self.assertTrue('no access to this view'
                        in self.publish_wiki_path('', wikiid='does_not_exist'))

    def test_wiki_page(self):
        self.assertTrue('content of subject2/content2'
                        in self.publish_wiki_path('subject2/content2'))

    def test_image(self):
        repo_path = self.datapath('vcwiki')
        fname = '120px-Crystal_Clear_app_kedit.png'
        image = self.publish_wiki_path(fname)
        with hglib.open(repo_path) as repo:
            image_file = repo.cat([osp.join(repo_path, fname)], rev='tip')
        self.assertEqual(image, image_file,
                         'wrong image content delivery')


class VCWikiEditControllerTC(VCWikiTC):

    def setup_database(self):
        super(VCWikiEditControllerTC, self).setup_database()
        with self.admin_access.web_request() as req:
            rql = 'Any X WHERE X is CWGroup, X name "managers"'
            managers = req.execute(rql).get_entity(0, 0)
            req.create_entity('CWPermission',
                              name=u'write',
                              label=u'vcwiki write',
                              require_group=managers,
                              reverse_require_permission=self.repoeid)
            req.cnx.commit()

    def check_edition_result(self, form, check_content_value=None, result=None):
        with self.admin_access.cnx() as cnx:
            vcwiki = cnx.entity_from_eid(self.vcwikieid)
            fpath = form['wikipath'] + '.%s' % vcwiki.content_file_extension
            vcontent = vcwiki.content(fpath)
            self.assertIsNotNone(vcontent,
                                 'could not find "%s" in current wiki' % fpath)
            if check_content_value is None:
                check_content_value = form['content']
            self.assertEqual(check_content_value, vcontent)

    def test_creation(self):
        self.check_edition_result(**self.vcwiki_edit(u'newsubject/dummy'))

    def test_edition(self):
        self.check_edition_result(**self.vcwiki_edit(u'subject2/content2'))

    def test_deletion(self):
        self.vcwiki_edit(u'subject2/content2', content=u'')
        with self.admin_access.cnx() as cnx:
            vcwiki = cnx.entity_from_eid(self.vcwikieid)
            vcontent = vcwiki.content(u'subject2/content2.rst',
                                      allow_deleted=True)
            self.assertFalse(vcontent)
            self.assertIsNotNone(vcontent)

    def test_preview_creation(self):
        result = self.vcwiki_edit(u'newsubject/preview',
                                  __action_preview = 'button_preview')['result']
        self.assertIn('new content', result)

    def test_preview_edition(self):
        with self.admin_access.cnx() as cnx:
            initial_content = cnx.entity_from_eid(self.vcwikieid).content(
                u'subject2/content2.rst')
        self.check_edition_result(
            check_content_value=initial_content,
            **self.vcwiki_edit(u'subject2/content2',
                               __action_preview = 'button_preview'))


if __name__ == '__main__':
    from logilab.common.testlib import unittest_main
    unittest_main()



# -*- coding: utf-8 -*-
# copyright 2013 LOGILAB S.A. (Paris, FRANCE), all rights reserved.
# contact http://www.logilab.fr -- mailto:contact@logilab.fr
#
# This program is free software: you can redistribute it and/or modify it under
# the terms of the GNU Lesser General Public License as published by the Free
# Software Foundation, either version 2.1 of the License, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
# details.
#
# You should have received a copy of the GNU Lesser General Public License along
# with this program. If not, see <http://www.gnu.org/licenses/>.

"""cubicweb-vcwiki views"""


from logilab.common.decorators import cachedproperty

from cubicweb.predicates import is_instance, match_form_params, objectify_predicate
from cubicweb.view import EntityView


@objectify_predicate
def has_write_perm_on_repo(cls, req, rset=None, entity=None, col=0, **kwargs):
    """Predicate to be complemented by is_instance('VCWiki'), that will return
    1 in the case the given VCWiki has a content repository on which current
    user as the update permission.
    """
    entity = entity or (rset and rset.get_entity(0, col))
    repo = entity.content_repo[0]
    rset = req.user.has_permission('write', contexteid=repo.eid)
    if rset:
        return 1
    return 0

def is_vcwiki_page(vcwiki):
    form = vcwiki._cw.form
    if 'wikipath' in form:
        vcpage_path = vcwiki.vcpage_path(form['wikipath'])
        if vcwiki.content(vcpage_path):
            return 1


class VCWikiView(EntityView):
    __abstract__ = True
    __select__ = (EntityView.__select__
                  & is_instance('VCWiki')
                  & match_form_params('wikipath'))
    consider_rev_parameter = False

    @property
    def vcontent(self):
        """ Return the contents of the current path in current vcwiki. This
        will consider the `rev` url parameter (which must contain a revision
        number) if the attribute `consider_rev_parameter` is set (which is not
        the default).
        """
        form = self._cw.form
        revnum = None
        if self.consider_rev_parameter:
            revnum = form.get('rev')
        vcpage_path = self.vcwiki.vcpage_path(form['wikipath'])
        return self.vcwiki.content(vcpage_path, revision=revnum)

    @property
    def vcpage_path(self):
        form = self._cw.form
        return self.vcwiki.vcpage_path(form['wikipath'])

    @cachedproperty
    def vcwiki(self):
        return self.cw_rset.get_entity(self.cw_row or 0, self.cw_col or 0)



# -*- coding: utf-8 -*-
# copyright 2013 LOGILAB S.A. (Paris, FRANCE), all rights reserved.
# contact http://www.logilab.fr -- mailto:contact@logilab.fr
#
# This program is free software: you can redistribute it and/or modify it under
# the terms of the GNU Lesser General Public License as published by the Free
# Software Foundation, either version 2.1 of the License, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
# details.
#
# You should have received a copy of the GNU Lesser General Public License along
# with this program. If not, see <http://www.gnu.org/licenses/>.

"""cubicweb-vcwiki web UI actions"""

from logilab.common.decorators import cachedproperty

from cubicweb.predicates import (is_instance, one_line_rset, match_form_params,
                                 score_entity)
from cubicweb.web.action import Action

from cubes.vcwiki import is_vfile_of_wiki
from cubes.vcwiki.views import is_vcwiki_page, has_write_perm_on_repo


class VCWikiAction(Action):
    __select__ = Action.__select__ & is_instance('VCWiki') & one_line_rset()

    @property
    def vcwiki(self):
        return self.cw_rset.get_entity(self.cw_row or 0, self.cw_col or 0)


class VCWikiViewAction(VCWikiAction):
    __regid__ = 'vcwiki.view_home'
    title = _('wiki homepage')
    category = 'moreactions'

    def url(self):
        return self.vcwiki.url


class VCWikiEntityViewAction(VCWikiAction):
    __regid__ = 'vcwiki.view_entity'
    __select__ = VCWikiAction.__select__ & match_form_params('wikipath')
    title = _('wiki entity')
    category = 'moreactions'

    def url(self):
        return self.vcwiki.absolute_url()


class VCWikiEditAction(VCWikiAction):
    __regid__ = 'edit' # override default edit action
    __select__ = (VCWikiAction.__select__
                  & match_form_params('wikipath')
                  & has_write_perm_on_repo())
    title = _('edit this page')
    category = 'mainactions'
    page_vid = 'vcwiki.edit_page'

    def actual_actions(self):
        """ Override default edit action for VCWiki instances, unless an
        existing wiki page is selected.
        """
        if is_vcwiki_page(self.vcwiki):
            yield self

    def url(self):
        return self.vcwiki.page_url(self._cw.form['wikipath'],
                vid=self.page_vid)


class VCWikiDeleteAction(VCWikiEditAction):
    __regid__ = 'delete' # override default delete action
    title = _('delete this page')
    category = 'mainactions'
    page_vid = 'vcwiki.delete_page'


class VCWikiViewHistoryAction(VCWikiEditAction):
    __regid__ = 'vcwiki.view_history'
    __select__ = (VCWikiAction.__select__
                  & match_form_params('wikipath'))
    title = _('history of this page')
    category = 'mainactions'
    page_vid = 'vcwiki.page_history'
    order = 1000



# -*- coding: utf-8 -*-
# copyright 2013 LOGILAB S.A. (Paris, FRANCE), all rights reserved.
# contact http://www.logilab.fr -- mailto:contact@logilab.fr
#
# This program is free software: you can redistribute it and/or modify it under
# the terms of the GNU Lesser General Public License as published by the Free
# Software Foundation, either version 2.1 of the License, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
# details.
#
# You should have received a copy of the GNU Lesser General Public License along
# with this program. If not, see <http://www.gnu.org/licenses/>.

"""cubicweb-vcwiki test utilities"""

from os import path as osp
from shutil import rmtree

from logilab.common.shellutils import unzip

# set up sys.path so we can find cubes.foo
import cubicweb.devtools

from cubes.vcsfile.testutils import VCSRepositoryTC, HGRCMixin

HERE = osp.dirname(__file__) or '.'

for repo in ('vcwiki',):
    repopath = osp.join(HERE, 'data', repo)
    if osp.exists(repopath):
        rmtree(repopath)
    unzip(osp.join(HERE, 'data', '%s.zip') % repo,  osp.join(HERE, 'data'))


class VCWikiTC(HGRCMixin, VCSRepositoryTC):
    _repo_path = u'vcwiki'

    def setup_database(self):
        """
        Create the vcwiki with write permission to managers from vcwiki.zip
        archive content.
        """
        with self.admin_access.cnx() as cnx:
            vcwiki = cnx.execute(
                'INSERT VCWiki W: W name "vcwiki", W content_repo R, '
                'W content_file_extension "rst" '
                'WHERE R is Repository').get_entity(0, 0)
            self.vcwikieid = vcwiki.eid
            self.repoeid = vcwiki.repository.eid
            # insert repository write permission for managers
            cnx.execute('INSERT CWPermission P: P name "write", P label "w p", '
                        'P require_group G, R require_permission P '
                        'WHERE R eid %(r)s, G name "managers"',
                        {'r': self.repoeid})
            cnx.commit()
        self.cleanup_repo()

    def cleanup_repo(self):
        """
        Restart from a fresh repository directory extracted from vcwiki.zip
        archive from the test/data directory.
        """
        if osp.exists(self.datapath('vcwiki')):
            rmtree(self.datapath('vcwiki'))
        unzip(self.datapath('vcwiki.zip'), self.datadir)

    def wiki_view(self, path, login=u'admin', **form):
        """
        Helper that returns the view of the wiki page at path `path`, with
        other keyword args passed as web form dict items.
        """
        with self.new_access(login).web_request() as req:
            req.form = {'wikipath': path}
            req.form.update(form)
            vcwiki = req.entity_from_eid(self.vcwikieid)
            return self.view('vcwiki.view_page', vcwiki.as_rset(), req=req)

    def vcwiki_edit(self, path, content=u'new content', **kwargs):
        """ Helper that edit the wiki page at path `path` with content
        `content`, other keyword args being used as items of the web form.
        Returns a dict with web form and controller publication result.
        """
        with self.admin_access.web_request() as req:
            req.form = {'vcwiki_eid': self.vcwikieid,
                        'wikipath': path,
                        'content': content,
                        'message': u'my commit',
                        'rev': 'tip'}
            req.form.update(kwargs)
            return {'form': req.form,
                    'result': self.ctrl_publish(req, ctrl='vcwiki.edit_page')}



# -*- coding: utf-8 -*-
# copyright 2013 LOGILAB S.A. (Paris, FRANCE), all rights reserved.
# contact http://www.logilab.fr -- mailto:contact@logilab.fr
#
# This program is free software: you can redistribute it and/or modify it under
# the terms of the GNU Lesser General Public License as published by the Free
# Software Foundation, either version 2.1 of the License, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
# details.
#
# You should have received a copy of the GNU Lesser General Public License along
# with this program. If not, see <http://www.gnu.org/licenses/>.

"""cubicweb-vcwiki actions' tests"""

from utils import VCWikiTC


class VCWikiActionsTC(VCWikiTC):

    def wiki_actions(self, login=u'admin', **form):
        with self.new_access(login).web_request() as req:
            req.form = form
            rset = req.execute('VCWiki X')
            return [a.__regid__
                    for a in rset.possible_actions()
                    if tuple(a.actual_actions())]

    def simple_user_wiki_actions(self, **form):
        with self.admin_access.cnx() as cnx:
            self.create_user(cnx, u'toto')
        return self.wiki_actions(u'toto', **form)

    def test_wiki_actions(self):
        actions = self.simple_user_wiki_actions()
        self.assertIn('vcwiki.view_home', actions)
        self.assertNotIn('vcwiki.view_entity', actions)

    def test_wiki_page_simple_user_actions(self):
        actions = self.simple_user_wiki_actions(wikipath=u'subject1/content1')
        self.assertNotIn('edit', actions)
        self.assertNotIn('delete', actions)
        self.assertIn('vcwiki.view_history', actions)
        self.assertIn('vcwiki.view_entity', actions)

    def test_wiki_existing_page_authorized_user_actions(self):
        actions = self.wiki_actions(wikipath=u'subject1/content1')
        self.assertIn('edit', actions)
        self.assertIn('delete', actions)
        self.assertIn('vcwiki.view_history', actions)
        self.assertIn('vcwiki.view_entity', actions)

    def test_wiki_non_existing_page_authorized_user_actions(self):
        actions = self.wiki_actions(wikipath=u'does_not_exist')
        self.assertNotIn('edit', actions)
        self.assertNotIn('delete', actions)
        self.assertNotIn('vcwiki.view_history', actions)
        self.assertIn('vcwiki.view_entity', actions)
        self.assertNotIn('vcwiki.version_content_view_entity', actions)


if __name__ == '__main__':
    from logilab.common.testlib import unittest_main
    unittest_main()



# -*- coding: utf-8 -*-
# copyright 2013 LOGILAB S.A. (Paris, FRANCE), all rights reserved.
# contact http://www.logilab.fr -- mailto:contact@logilab.fr
#
# This program is free software: you can redistribute it and/or modify it under
# the terms of the GNU Lesser General Public License as published by the Free
# Software Foundation, either version 2.1 of the License, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
# details.
#
# You should have received a copy of the GNU Lesser General Public License along
# with this program. If not, see <http://www.gnu.org/licenses/>.

"""cubicweb-vcwiki wiki display related views"""

from logilab.mtconverter import xml_escape

from cubicweb.predicates import is_instance, score_entity
from cubicweb.tags import a, div
from cubicweb.uilib import cut
from cubicweb.web import stdmsgs
from cubicweb.web.formfields import StringField
from cubicweb.web.formwidgets import Button, SubmitButton, HiddenInput
from cubicweb.web.views.forms import FieldsForm
from cubicweb.web.views.baseviews import (InContextView, OneLineView,
                                          OutOfContextView)


from cubes.vcwiki import is_vcontent_of_wiki
from cubes.vcwiki.views import VCWikiView, has_write_perm_on_repo


class VCWikiViewPage(VCWikiView):
    """View page when the user cannot edit"""
    __regid__ = 'vcwiki.view_page'
    __select__ = VCWikiView.__select__ & ~has_write_perm_on_repo()
    consider_rev_parameter = True

    def empty_page_message(self):
        """Return the HTML snippet indicating that the page does not exist."""
        return self._cw._('This wiki page does not exist.')

    def page_title(self):
        if self.vcontent is not None:
            name = self.vcwiki.display_name(self.vcpage_path)
        else:
            name = self._cw._('New wiki page')
        return u'%s - %s' % (self.vcwiki.dc_title(), name)

    def entity_call(self, entity):
        if not self.vcontent:
            self.w(div(self.empty_page_message(), Class='stateMessage'))
        else:
            from logilab.mtconverter import TransformData
            from cubicweb.uilib import soup2xhtml
            from cubicweb.mttransforms import ENGINE
            context = self.vcwiki
            data = TransformData(self.vcontent, 'text/rest', 'utf-8', appobject=context)
            new_html_content = soup2xhtml(ENGINE.convert(data, 'text/html').decode(), 'utf-8')
            self.w(new_html_content)


class VCWikiViewPageCanEdit(VCWikiViewPage):
    """View page, when the user can edit"""
    __select__ = VCWikiView.__select__ & has_write_perm_on_repo()

    def empty_page_message(self):
        """Return the HTML snippet indicating that the page does not exist,
        with an invite link to create it.
        """
        basemsg = super(VCWikiViewPageCanEdit, self).empty_page_message()
        url = self.vcwiki.page_url(self._cw.form['wikipath'],
                                   vid='vcwiki.edit_page')
        link = a(self._cw._('Create this wiki page?'), href=xml_escape(url))
        return u' '.join([basemsg, link])


class VCWikiBaseItemViewMixIn(object):

    def cell_call(self, row, col, **kwargs):
        """Method used to override the default link target of the different one
        line views (oneline, incontext, outofcontext).
        """
        entity = self.cw_rset.get_entity(row, col)
        desc = cut(entity.dc_description(), 50)
        title = cut(entity.dc_title(),
                    self._cw.property_value('navigation.short-line-size'))
        self.w(u'<a href="%s" title="%s">%s</a>' % (
                xml_escape(entity.url), xml_escape(desc),
                xml_escape(title)))


class WikiOneLineView(VCWikiBaseItemViewMixIn, OneLineView):
    __select__ = OneLineView.__select__ & is_instance('VCWiki')


class WikiInContextView(VCWikiBaseItemViewMixIn, InContextView):
    __select__ = InContextView.__select__ & is_instance('VCWiki')


class WikiOutOfContextView(VCWikiBaseItemViewMixIn, OutOfContextView):
    __select__ = OutOfContextView.__select__ & is_instance('VCWiki')



# copyright 2014 LOGILAB S.A. (Paris, FRANCE), all rights reserved.
# contact http://www.logilab.fr -- mailto:contact@logilab.fr
#
# This program is free software: you can redistribute it and/or modify it under
# the terms of the GNU Lesser General Public License as published by the Free
# Software Foundation, either version 2.1 of the License, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
# details.
#
# You should have received a copy of the GNU Lesser General Public License along
# with this program. If not, see <http://www.gnu.org/licenses/>.

"""cubicweb-vcwiki command plugins"""

import hglib

from logilab.common.shellutils import ASK

from cubicweb import cwconfig
from cubicweb.server.repository import Repository
from cubicweb.server.utils import manager_userpasswd
from cubicweb.toolsutils import Command, CONNECT_OPTIONS
from cubicweb.cwctl import CWCTL


class CreateVCWiki(Command):
    """Create and setup a VCWiki with associated repository.

    Write permissions for 'users' and 'managers' are granted on the newly
    created Repository.
    If the Mercurial repository is not initial (or does not exists), the
    command will do it.
    """
    name = 'new-vcwiki'
    arguments = '<instance> <wikiname> <repopath>'
    min_args = 3
    max_args = 3
    options = CONNECT_OPTIONS

    def run(self, args):
        appid = args.pop(0)
        config = cwconfig.instance_configuration(appid)
        user, password = self.config.user, self.config.password
        if not password:
            user, password = manager_userpasswd(user=user, msg='')
        repo = Repository(config, TasksManager())
        session = repo.new_session(user, password=password)
        self.cnx = session.new_cnx()
        wikiname, repopath = args
        with self.cnx:
            try:
                self.add_vcwiki(wikiname, repopath)
            except:
                self.cnx.rollback()
                raise
            else:
                self.cnx.commit()
                self.setup_hgrepo(repopath)

    def add_vcwiki(self, wikiname, repopath):
        """Add a VCWiki and corresponding Repository, with write permission
        for 'users' and 'managers'"""
        repos = self.cnx.create_entity(
            'Repository', title=u'Repository for %s' % wikiname,
            path=unicode(repopath), type=u'mercurial')
        wiki = self.cnx.create_entity(
            'VCWiki', name=unicode(wikiname), content_file_extension=u'rst',
            content_repo=repos)
        perm = self.cnx.create_entity(
            'CWPermission', name=u'write',
            label=u'write permission for %s repository' % wikiname)
        for group in (u'users', u'managers'):
            group = self.cnx.find('CWGroup', name=group).one()
            perm.cw_set(require_group=group)
        repos.cw_set(require_permission=perm)

    def setup_hgrepo(self, repopath):
        """Check that the Mercurial repository is operational."""
        try:
            hglib.open(repopath).close() # Test existence, then close it.
        except hglib.error.ServerError:
            if ASK.confirm('The Mercurial repository for the Wiki does not seem to '
                           'exist, create it?'):
                hglib.init(repopath)
                print 'Repository %s initialized' % repopath


CWCTL.register(CreateVCWiki)



import posixpath as posixp

from docutils import nodes, utils
from docutils.parsers.rst.roles import register_canonical_role, set_classes


def wiki_page_reference_role(role, rawtext, text, lineno, inliner,
                             options={}, content=[]):
    text = text.strip()
    try:
        wikipath, rest = text.split(u':', 1)
    except:
        wikipath, rest = text, text
    vcwiki = inliner.document.settings.context
    if 'wikipath' not in vcwiki._cw.form:
        return [nodes.Text(rest)], []
    my_path = vcwiki.vcpage_path(vcwiki._cw.form['wikipath'])
    # Allow users to link to a page with file extension
    ext = '.' + vcwiki.content_file_extension
    if wikipath.endswith(ext):
        wikipath = wikipath[:-len(ext)]
    # Return url and markup options
    if not wikipath.startswith('/'):
        vcpath = posixp.join(posixp.dirname(my_path), wikipath)
    else:
        vcpath = wikipath[1:]
    vcpath = posixp.normpath(vcpath)
    set_classes(options)
    if vcwiki.content(vcwiki.vcpage_path(vcpath)) is None:
        options['classes'] = ['doesnotexist']
    else:
        options.pop('classes', None)
    return [nodes.reference(rawtext, utils.unescape(rest),
                            refuri=vcwiki.page_url(vcpath),
                            **options)], []


register_canonical_role('wiki', wiki_page_reference_role)



"""cubicweb-vcwiki application package

Version controlled wiki component for the CubicWeb framework
"""


def is_vfile_of_wiki(entity):
    return bool(entity._cw.execute(
            "Any W WHERE W content_repo R, X eid %(x)s, X from_repository R, "
            "W content_file_extension EXT, X name ~= ('%.' + EXT)",
            {'x': entity.eid}))


def is_vcontent_of_wiki(entity):
    return bool(entity._cw.execute(
            "Any W WHERE W content_repo R, X eid %(x)s, "
            "X content_for F, F from_repository R, "
            "W content_file_extension EXT, F name ~= ('%.' + EXT)",
            {'x': entity.eid}))



# -*- coding: utf-8 -*-
# copyright 2013 LOGILAB S.A. (Paris, FRANCE), all rights reserved.
# contact http://www.logilab.fr -- mailto:contact@logilab.fr
#
# This program is free software: you can redistribute it and/or modify it under
# the terms of the GNU Lesser General Public License as published by the Free
# Software Foundation, either version 2.1 of the License, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
# details.
#
# You should have received a copy of the GNU Lesser General Public License along
# with this program. If not, see <http://www.gnu.org/licenses/>.

"""cubicweb-vcwiki entity tests"""

from logilab.common.decorators import cachedproperty

from cubicweb.web.views.ibreadcrumbs import IBreadCrumbsAdapter

from utils import VCWikiTC


class VCWikiContentTC(VCWikiTC):

    def vcwiki_content(self, fpath, allow_deleted=False):
        with self.admin_access.web_request() as req:
            vcwiki = req.entity_from_eid(self.vcwikieid)
            return vcwiki.content(fpath, allow_deleted=allow_deleted)
    
    def test_simple(self):
        self.assertIsNotNone(self.vcwiki_content('hello.rst'))

    def test_directory(self):
        self.assertEqual('index file of subject1 folder\n',
                         self.vcwiki_content('subject1.rst'))

    def test_root(self):
        self.assertEqual('index file of the root folder\n',
                         self.vcwiki_content('index.rst'))

    def test_does_not_exist(self):
        self.assertIsNone(self.vcwiki_content('does_not_exist.rst'))

    def test_deleted(self):
        self.assertIsNone(self.vcwiki_content('deleted.rst'))
        content = self.vcwiki_content('deleted.rst', allow_deleted=True)
        self.assertIs(False, content)

    def test_index_if_no_path(self):
        """ Base wiki's url must have a trailing '/' so that the wiki's index
        file can contain relative links.
        """
        with self.admin_access.web_request() as req:
            vcwiki = req.entity_from_eid(self.vcwikieid)
            self.assertEqual(vcwiki.page_urlpath(''), u'wiki/%s/' % vcwiki.name)


class VCWikiBreadcrumbsTC(VCWikiTC):

    @cachedproperty
    def vcwiki_url(self):
        with self.admin_access.web_request() as req:
            return req.entity_from_eid(self.vcwikieid).url
    
    def wiki_breadcrumbs(self, wikipath):
        with self.admin_access.web_request() as req:
            req.form = {'wikipath': wikipath}
            vcwiki = req.entity_from_eid(self.vcwikieid)
            return vcwiki.cw_adapt_to('IBreadCrumbs').breadcrumbs()

    def test_standard_breadcrumbs(self):
        """If we are not displaying a wiki page but the VCWiki instance itself,
        we should not use the custom breadcrumbs adapter"""
        with self.admin_access.web_request() as req:
            vcwiki = req.entity_from_eid(self.vcwikieid)
            self.assertEqual(vcwiki.cw_adapt_to('IBreadCrumbs').__class__,
                             IBreadCrumbsAdapter)

    def test_root(self):
        self.assertListEqual([(self.vcwiki_url, u'vcwiki')],
                             self.wiki_breadcrumbs(''))

    def test_simple(self):
        self.assertListEqual([(self.vcwiki_url, u'vcwiki'),
                              'hello'],
                             self.wiki_breadcrumbs('hello'))

    def test_subdir_file(self):
        self.assertListEqual([(self.vcwiki_url, u'vcwiki'),
                              (self.vcwiki_url + 'subject1', u'subject1'),
                              u'content1'],
                             self.wiki_breadcrumbs('subject1/content1'))

    def test_directory_has_no_index(self):
        self.assertListEqual([(self.vcwiki_url, u'vcwiki'),
                              'subject2',
                              'content2'],
                             self.wiki_breadcrumbs('subject2/content2'))

    def test_does_not_exist(self):
        self.assertListEqual([(self.vcwiki_url, u'vcwiki')],
                             self.wiki_breadcrumbs('does_not_exist'))


if __name__ == '__main__':
    from logilab.common.testlib import unittest_main
    unittest_main()



import os

import hglib

from cubicweb import Binary
from cubicweb.server import Service

from cubes.vcsfile import bridge


class RevisionDiffService(Service):
    """ Return the diff between two revisions.
    """

    __regid__ = 'vcwiki.export-rev-diff'

    def call(self, repo_eid, path, rev1, rev2):
        repo = self._cw.entity_from_eid(repo_eid)
        adapter = repo.cw_adapt_to('VCSRepo')
        filepath = os.path.join(adapter.path, path)
        try:
            with adapter.hgrepo() as hgrepo:
                out_diff = hgrepo.diff(files=[filepath.encode(repo.encoding)],
                                       revs=[rev1, rev2], git=True, unified=5)
            return out_diff
        except hglib.error.ServerError:
            return None



# -*- coding: utf-8 -*-
# copyright 2013 LOGILAB S.A. (Paris, FRANCE), all rights reserved.
# contact http://www.logilab.fr -- mailto:contact@logilab.fr
#
# This program is free software: you can redistribute it and/or modify it under
# the terms of the GNU Lesser General Public License as published by the Free
# Software Foundation, either version 2.1 of the License, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
# details.
#
# You should have received a copy of the GNU Lesser General Public License along
# with this program. If not, see <http://www.gnu.org/licenses/>.

"""cubicweb-vcwiki schema"""


from yams.buildobjs import EntityType, SubjectRelation, String
from yams.constraints import SizeConstraint


class VCWiki(EntityType):
    name = String(required=True, unique=True, maxsize=128)
    content_repo = SubjectRelation('Repository', cardinality='1?', inlined=True)
    content_file_extension = String(required=True, maxsize=16,
                                    constraints=[SizeConstraint(min=1)],
                                    description=_(
            'file extension of vcwiki pages as stored in the repository,'
            ' without leading point (".").'
            ' Examples: "rst", "html"'))


def post_build_callback(schema):
    return # XXX
    # Make VersionContent data attribute fulltextindexed
    schema['VersionContent'].rdef('data').fulltextindexed = True



# -*- coding: utf-8 -*-
# copyright 2013 LOGILAB S.A. (Paris, FRANCE), all rights reserved.
# contact http://www.logilab.fr -- mailto:contact@logilab.fr
#
# This program is free software: you can redistribute it and/or modify it under
# the terms of the GNU Lesser General Public License as published by the Free
# Software Foundation, either version 2.1 of the License, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
# details.
#
# You should have received a copy of the GNU Lesser General Public License along
# with this program. If not, see <http://www.gnu.org/licenses/>.

"""cubicweb-vcwiki entity's classes"""

import os.path as osp

import mimetypes
mimetypes.add_type('text/rest', '.rst')

from logilab.common.decorators import cached, cachedproperty

from cubicweb.entities import AnyEntity
from cubicweb.entities.adapters import IFTIndexableAdapter
from cubicweb.predicates import is_instance, match_form_params
from cubicweb.web.views.ibreadcrumbs import IBreadCrumbsAdapter

from cubes.vcwiki import is_vfile_of_wiki


def split_path(path):
    """ Return a tuple (file directory, file name) deduced from the `path`
    parameter. This method uses '/' as a separator between file directory
    and path.
    """
    filedir, sep, name = path.rpartition('/')
    return filedir, name


class VCWiki(AnyEntity):
    __regid__ = 'VCWiki'

    @property
    def repository(self):
        """ Short accessor to the wiki's repository. """
        return self.content_repo[0]

    @property
    def urlpath(self):
        """ Path part of the wiki's URL. """
        return 'wiki/%s/' % self._cw.url_quote(self.name)

    @property
    def url(self):
        """ URL of the current wiki. """
        return self._cw.build_url(self.urlpath)

    def page_urlpath(self, path):
        """ Path part of a wiki page's URL. """
        return self.urlpath + self._cw.url_quote(path, safe='/')

    def page_url(self, path, **kwargs):
        """ Helper to build the URL of the wiki page's that has `path` path.
        Use additional keyword args to specify other URL parameters, like "vid".
        """
        return self._cw.build_url(self.page_urlpath(path), **kwargs)

    def vcpage_path(self, wikipath):
        """ Transform `wikipath`, a url path part of the web wiki, into the path
        of the corresponding wiki page in the mercurial repository, adding the 
        vcwiki's content file extension.
        """
        wikipath = wikipath or 'index' # handle root url path
        return wikipath + '.' + self.content_file_extension

    @cached
    def current_changeset(self):
        rset = self.repository.cw_adapt_to('VCSRepo').log_rset(revrange='tip')
        if not rset:
            return '0' * 12
        return next(rset.entities()).changeset

    def content(self, path, revision=None, allow_deleted=False):
        """ Return the content of the file with given path if it exists in the
        given revision (or latest one if None) of current wiki's repository.

        If the `allow_deleted` parameter is True, a DeletedVersionContent can be
        returned, if any. Otherwise, deleted content will lead to a None value.

        If nothing was found, return None.
        """
        if revision is None:
            revision = self.current_changeset()
        try:
            return self.repository.cw_adapt_to('VCSRepo').cat(rev=revision, path=path)
        except Exception:
            if allow_deleted and self.repository.cw_adapt_to('VCSRepo').file_deleted(path):
                return False
            return None

    def __contains__(self, path):
        return path in self.repository.cw_adapt_to('VCSRepo')

    def display_name(self, path):
        """Return a displayable name for a page -normally part of the
        wiki's content-, build by stripping the trailing vcwiki's extension,
        if present."""
        name = osp.basename(path)
        ext = '.' + self.content_file_extension
        if name.endswith(ext):
            name = name[:-len(ext)]
        return name

    @cachedproperty
    def content_mimetype(self):
        return mimetypes.types_map.get('.' + self.content_file_extension,
                                       'text/plain')


class VCWikiBreadcrumbs(IBreadCrumbsAdapter):
    """ Breadcrumbs adapter for VCWiki entities to expose the directory
    structure of the wiki.
    """
    __select__ = (IBreadCrumbsAdapter.__select__ & is_instance('VCWiki') &
                  match_form_params('wikipath'))

    @property
    def vcpath(self):
        return self.entity.vcpage_path(self._cw.form['wikipath'])

    def breadcrumbs(self, view=None, recurs=None):
        """Hierarchy of wiki pages following its repository's directory-like
        structure. Return a link to the index page of the parent directories.
        """
        path = []
        vcwiki = self.entity
        wikipath = self._cw.form['wikipath']
        if wikipath and self.vcpath in vcwiki: # skip root path and 404
            dirs = self.vcpath.split('/')[:-1]
            path.append(wikipath.split('/')[-1])
            while dirs:
                dirpath = u'/'.join(dirs)
                dirname = dirs.pop()
                vcontent = vcwiki.content(vcwiki.vcpage_path(dirpath))
                if vcontent is not None:
                    path.append((vcwiki.page_url(dirpath), dirname))
                else:
                    path.append(dirname)
        path.append((vcwiki.url, vcwiki.name))
        path.reverse()
        return path



# -*- coding: utf-8 -*-
# copyright 2013 LOGILAB S.A. (Paris, FRANCE), all rights reserved.
# contact http://www.logilab.fr -- mailto:contact@logilab.fr
#
# This program is free software: you can redistribute it and/or modify it under
# the terms of the GNU Lesser General Public License as published by the Free
# Software Foundation, either version 2.1 of the License, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
# details.
#
# You should have received a copy of the GNU Lesser General Public License along
# with this program. If not, see <http://www.gnu.org/licenses/>.

"""cubicweb-vcwiki urlrewrite related code"""


from cubicweb.web.views.urlrewrite import SchemaBasedRewriter, rgx, rgx_action


class VCWikiURLRewriter(SchemaBasedRewriter):
    """handle path with the form::

        wiki/<wikiid>   -> view wiki page, wikiid can contain /

    Fall back to the `wiki` controller in any cases. The latter will handle
    both existent and non-existent cards.
    """
    priority = 10
    rules = [(rgx('/wiki/(?P<wikiid>[^/]+)(?P<wikipath>.+)?'),
              rgx_action(controller='wiki', formgroups=('wikiid', 'wikipath'))),
            ]



# -*- coding: utf-8 -*-
# copyright 2013 LOGILAB S.A. (Paris, FRANCE), all rights reserved.
# contact http://www.logilab.fr -- mailto:contact@logilab.fr
#
# This program is free software: you can redistribute it and/or modify it under
# the terms of the GNU Lesser General Public License as published by the Free
# Software Foundation, either version 2.1 of the License, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
# details.
#
# You should have received a copy of the GNU Lesser General Public License along
# with this program. If not, see <http://www.gnu.org/licenses/>.

"""cubicweb-vcwiki controllers"""

import json
from io import BytesIO
import mimetypes

from logilab.common.decorators import cachedproperty
from logilab.mtconverter import guess_mimetype_and_encoding, TransformData

from cubicweb import Binary
from cubicweb.uilib import soup2xhtml
from cubicweb.mttransforms import ENGINE
from cubicweb.predicates import is_instance, match_form_params
from cubicweb.web import Redirect, ValidationError
from cubicweb.web.views.basecontrollers import ViewController
from cubicweb.web.views.editcontroller import EditController

from cubes.preview.views.controllers import PreviewControllerMixin

from cubes.vcwiki.entities import split_path

class VCWikiViewController(ViewController):
    __regid__ = 'wiki'
    __select__ = ViewController.__select__ & match_form_params('wikiid', 'wikipath')

    def publish(self, rset=None):
        if self.vcwiki is None:
            self._cw.form['vid'] = 'noresult'
            return super(VCWikiViewController, self).publish()

        self.normalize_path()
        mimetype, vcontent = self.get_downloadable_content()
        if mimetype and not mimetype.startswith('text/'):
            self._cw.set_content_type(mimetype)
            return vcontent
        self._cw.form.setdefault('vid', 'vcwiki.view_page')
        rset = self.vcwiki.as_rset()
        return super(VCWikiViewController, self).publish(rset)

    @cachedproperty
    def vcwiki(self):
        """ Return VCWiki instance designated by the URL or None."""
        rset = self._cw.execute('VCWiki W WHERE W name %(n)s',
                                {'n': self._cw.form.pop('wikiid')})
        if rset:
            return rset.get_entity(0, 0)

    def normalize_path(self):
        """ Normalize the web form's `path` parameter, handling the None case
        (== root path), and removing leading and trailing slashes, if any.
        """
        form = self._cw.form
        form['wikipath'] = (form['wikipath'] or u'').lstrip('/').rstrip('/')

    def get_downloadable_content(self):
        """ Return a boolean telling if the wiki content (if any) that matches
        the web form's wiki path `wikipath` is binary or not, and the
        matching VersionContent instance (or None if none match the given path).

        First try to get the content as a textual content, adding the vcwiki's
        content file extension to the web form given wiki path.

        If this fails (no content exists in the wiki's repository at this path)
        consider the path with no extension.
        """
        mimetype = 'text/'
        for vcfile_path in (self.vcwiki.vcpage_path(self._cw.form['wikipath']),
                            self._cw.form['wikipath']):
            vcontent = self.vcwiki.content(vcfile_path)
            if vcontent is not None:
                mimetype = mimetypes.guess_type(vcfile_path)[0]
                break
        return mimetype, vcontent


class VCWikiEditController(PreviewControllerMixin, EditController):
    __regid__ = 'vcwiki.edit_page'
    __select__ = match_form_params('vcwiki_eid', 'wikipath', 'content', 'message')

    def create_new_revision(self, vpath, content, msg):
        """ Create a new revision of the vcwiki with commit msg `msg`.
        If content is None, the versioned file with path `path` (if any) is
        deleted, otherwise its content is updated with `content`.
        When `content` if empty and the path points to a non yet existing file,
        an error is raised.
        """
        # handle the above-described error case
        if content is None and vpath not in self.vcwiki.repository.cw_adapt_to('VCSRepo').manifest(rev='tip'):
            raise ValidationError(self.vcwiki.eid, {'content': self._cw._(
                'can not set an empty content to a new wiki page')})
        if not content:
            added = ()
            deleted = (vpath,)
        else:
            content = content.encode('utf-8')
            added = ((vpath, BytesIO(content)),)
            deleted = ()
        # create the new revision
        author = self._cw.user.dc_title()
        if self._cw.user.primary_email:
            author += u' <%s>' % self._cw.user.primary_email[0].address
        new_rev = self.vcwiki.repository.make_revision(
            msg=msg.encode('utf-8'), author=author.encode('utf-8'),
            added=added, deleted=deleted, parentrev='tip').changeset
        repo = self.vcwiki.repository
        if repo.source_url:
            with repo.cw_adapt_to('VCSRepo').hgrepo() as client:
                client.push(dest=repo.source_url, rev=new_rev, force=True)

    @cachedproperty
    def vcwiki(self):
        return self._cw.entity_from_eid(int(self._cw.form['vcwiki_eid']))

    def _default_publish(self):
        form = self._cw.form
        msg = form.get('message') or u'no commit message'
        vcfile_path = self.vcwiki.vcpage_path(form['wikipath'])
        self.create_new_revision(vcfile_path, form['content'], msg)

    def publish(self, rset=None):
        """ Use all EditController machinery but the HTTP Redirect response, to
        be transformed into a javascript redirect. """
        try:
            return super(VCWikiEditController, self).publish(rset)
        except Redirect as exc:
            return self._javascript_redirect(exc.location)
        except ValidationError as ex:
            return self._javascript_error(ex)

    def _javascript_redirect(self, url):
        """ Javascript redirect response from a child iframe. """
        return ('<script type="text/javascript">'
                'window.parent.location.replace(%s);'
                '</script>' % json.dumps(url))

    def _javascript_error(self, exception):
        self._cw.set_content_type('text/html')
        return """<script type="text/javascript">
 window.parent.handleFormValidationResponse('form', null, null, %s, null);
</script>""" % (json.dumps((False, (None, exception.errors), None)))

    def _action_preview(self):
        form = self._cw.form
        vcfile_path = self.vcwiki.vcpage_path(form['wikipath'])
        data = form['content'].encode(self._cw.encoding)
        format, encoding = guess_mimetype_and_encoding(
            encoding=self._cw.encoding, filename=vcfile_path, data=data)
        data = TransformData(data, format, encoding, appobject=self.vcwiki)
        new_html_content = soup2xhtml(
            ENGINE.convert(data, 'text/html').decode(), encoding)
        domid = form.get('__domid', 'form')
        return self._preview_trampoline(domid, new_html_content, 'inline')



# -*- coding: utf-8 -*-
# copyright 2013 LOGILAB S.A. (Paris, FRANCE), all rights reserved.
# contact http://www.logilab.fr -- mailto:contact@logilab.fr
#
# This program is free software: you can redistribute it and/or modify it under
# the terms of the GNU Lesser General Public License as published by the Free
# Software Foundation, either version 2.1 of the License, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
# details.
#
# You should have received a copy of the GNU Lesser General Public License along
# with this program. If not, see <http://www.gnu.org/licenses/>.

"""cubicweb-vcwiki wiki display related views"""

from functools import partial

from logilab.mtconverter import xml_escape

from logilab.common.decorators import cachedproperty

from cubicweb.tags import a, div, input, span
from cubicweb.predicates import is_instance, match_form_params, score_entity
from cubicweb.web import stdmsgs
from cubicweb.web.form import Form
from cubicweb.web.formfields import StringField
from cubicweb.web.formwidgets import Button, SubmitButton, HiddenInput, Radio
from cubicweb.web.views.forms import FieldsForm
from cubicweb.web.views.formrenderers import HTableFormRenderer

from cubes.vcwiki.views import is_vcwiki_page, VCWikiView, has_write_perm_on_repo
from cubes.preview.views.forms import PreviewFormMixin


class VCWikiPageForm(PreviewFormMixin, FieldsForm):
    __abstract__ = True
    _default_form_action_path = 'vcwiki.edit_page'
    formtype = 'main'
    form_buttons = [SubmitButton(),
                    Button(stdmsgs.BUTTON_CANCEL, cwaction='cancel')]
    # Hidden fields
    vcwiki_eid = StringField('vcwiki_eid', widget=HiddenInput)
    path = StringField('wikipath', widget=HiddenInput)
    redirect = StringField('__redirectpath', widget=HiddenInput)
    # Commit message (visible) field
    message = StringField('message', label=_('commit message'))

    def redirect_path(self):
        return self.vcwiki.page_urlpath(unicode(self._cw.form['wikipath']))

    def __init__(self, *args, **kwargs):
        self.vcwiki = kwargs.pop('entity')
        self.vcontent = kwargs.pop('vcontent')
        super(VCWikiPageForm, self).__init__(*args, **kwargs)
        fbn = self.field_by_name
        fbn('vcwiki_eid').value = unicode(self.vcwiki.eid)
        fbn('wikipath').value = self.path
        fbn('__redirectpath').value = self.redirect_path()


class VCWikiEditPageView(VCWikiView):
    __regid__ = 'vcwiki.edit_page'
    __select__ = VCWikiView.__select__ & has_write_perm_on_repo()
    form_renderer_id = 'default'
    consider_rev_parameter = True

    @property
    def formid(self):
        """ Property used to help inheritance. """
        return self.__regid__

    def entity_call(self, entity):
        form = self._cw.vreg['forms'].select(
            self.formid, self._cw, form_renderer_id=self.form_renderer_id,
            entity=self.vcwiki, vcontent=self.vcontent)
        form.render(w=self.w)


class VCWikiEditPageForm(VCWikiPageForm):
    __regid__ = 'vcwiki.edit_page'
    __select__ = VCWikiPageForm.__select__ & has_write_perm_on_repo()

    @property
    def content_value(self):
        if self.vcontent is None:
            return u''
        return unicode(self.vcontent, 'utf-8')

    def __init__(self, *args, **kwargs):
        super(VCWikiEditPageForm, self).__init__(*args, **kwargs)
        field = StringField(_('content'), required=True,
                            value=self.content_value)
        self.fields.insert(0, field)


class VCWikiDeletePageView(VCWikiEditPageView):
    __regid__ = 'vcwiki.delete_page'
    __select__ = VCWikiEditPageView.__select__ & score_entity(is_vcwiki_page)


class VCWikiDeletePageForm(VCWikiPageForm):
    __regid__ = 'vcwiki.delete_page'
    content = StringField(_('content'), value=u'', widget=HiddenInput)

    def redirect_path(self):
        return self.vcwiki.urlpath


class VCWikiPageHistory(VCWikiView):
    __regid__ = 'vcwiki.page_history'
    __select__ = VCWikiView.__select__ & score_entity(is_vcwiki_page)

    @cachedproperty
    def revisions(self):
        return list(self.vcwiki.repository.cw_adapt_to('VCSRepo').log_rset(self.vcpage_path).entities())

    def page_title(self):
        return (self._cw._('%(vcwiki)s - History of %(wikipath)s')
                % {'vcwiki': self.vcwiki.dc_title(),
                   'wikipath': self._cw.form['wikipath'] or self._cw._('root index')})

    def intro(self):
        self.w(u'<div class="section"><h1>%s</h1>' % self.page_title())
        self.w(u'<p>%s</p>' % self._cw._('Choose two revisions in the form '
                                         'below to compare them.'))
        self.w(u'<p>%s</p>' % self._cw._('You may view the page as it was at a '
                                         'given revision by clicking on it.'))
        self.w(u'<p>%s</p>' % self._cw._('You may also revert the page content '
                                         'to its value using the revert link.'))
        self.w(u'</div>')

    def rev1_html(self, name='rev1', default_value=None, reverse_tags=False):
        html = []
        selected = self._cw.form.get(name) or default_value
        wikipath = self._cw.form['wikipath']
        for rev in self.revisions:
            a_content = rev.changeset or ('0' * 12)
            a_attrs = {'title': rev.description,
                       'href': self.vcwiki.page_url(wikipath, rev=rev.changeset)}
            i_attrs = {'name': name, 'type': 'radio', 'value': rev.changeset}
            if selected == rev.changeset:
                i_attrs['checked'] = 'checked'
            tags = [a(a_content, **a_attrs), input(**i_attrs)]
            if reverse_tags:
                tags.reverse()
            div_attrs = {}
            if rev.changeset == self.vcwiki.current_changeset():
                div_attrs['class'] = 'current'
            html.append(div(''.join(tags), **div_attrs))
        return ''.join(html)

    def rev2_html(self):
        return self.rev1_html('rev2', self.vcwiki.current_changeset(), True)

    def rev_date_html(self):
        fdate = partial(self._cw.format_date, time=True)
        return u''.join([div(fdate(rev.creation_date))
                         for rev in self.revisions])

    def rev_revert_html(self):
        html = []
        wikipath = self._cw.form['wikipath']
        for rev in self.revisions:
            div_content = u''
            if rev.eid != self.revisions[0].eid:
                msg = 'revert to revision %s' % rev.changeset
                href = self.vcwiki.page_url(wikipath, vid='vcwiki.edit_page',
                                            rev=rev.changeset, message=msg)
                a_content = self._cw._('Revert to revision %s') % rev.changeset
                div_content = a(a_content, href=href)
            html.append(div(div_content))
        return u''.join(html)

    def comparison_form(self):
        label = (self._cw._('Compare'), 'OK_ICON')
        button_html = SubmitButton(label=label).render(self)
        ctx = {'action': self.vcwiki.page_url(self._cw.form['wikipath']),
               'vid': xml_escape(self.__regid__),
               'rev1': self.rev1_html(),
               'rev2': self.rev2_html(),
               'date': self.rev_date_html(),
               'revert': self.rev_revert_html(),
               'h_rev1': xml_escape(self._cw._('Rev. 1')),
               'h_rev2': xml_escape(self._cw._('Rev. 2')),
               'h_date': xml_escape(self._cw._('Date')),
               'h_revert': xml_escape(self._cw._('Revert to an old rev.')),
               'button': button_html,
               }
        self.w(u'<form action="%(action)s" method="get">'
               u'<input type="hidden" name="vid" value="%(vid)s"/>'
               u'<table class="listing table table-bordered table-condensed diff"><tbody>'
               u'<tr>'
               u'<th>%(h_rev1)s</th>'
               u'<th>%(h_rev2)s</th>'
               u'<th>%(h_date)s</th>'
               u'<th>%(h_revert)s</th>'
               u'</tr>'
               u'<tr>'
               u'<td class="rev1">%(rev1)s</td>'
               u'<td class="rev2">%(rev2)s</td>'
               u'<td class="date">%(date)s</td>'
               u'<td class="revert">%(revert)s</td>'
               u'</tr>'
               u'</tbody></table>'
               u'%(button)s'
               u'</form>' % ctx)

    def get_revision(self, repo, cs):
        return self._cw.execute(
            'Any R WHERE R changeset %(rev)s, R from_repository %(repo)s',
            {'repo': repo.eid, 'rev': cs}).get_entity(0, 0)

    def _write_msg(self, msg):
            self.w(u'<div class="stateMessage">%s</div>' % self._cw._(msg))

    def display_diff(self):
        form = self._cw.form
        if 'rev1' not in form or 'rev2' not in form:
            return
        repo = self.vcwiki.repository
        diff = None
        if repo.type == 'mercurial':
            rev1 = self.get_revision(repo, form['rev1'])
            rev2 = self.get_revision(repo, form['rev2'])
            diff = self._cw.call_service(
                'vcwiki.export-rev-diff', repo_eid=repo.eid,
                path=self.vcpage_path,
                rev1=rev1.changeset, rev2=rev2.changeset)
        self.w(u'<div class="section">')
        if diff is None:
            self._write_msg(_('Could not compute diff.'))
        elif diff == '':
            self._write_msg(_('Files are identical.'))
        else:
            self.w(u'<div class="content">')
            transformer = rev1._cw_mtc_transform
            html = transformer(diff, 'text/x-diff', 'text/annotated-html', 'utf8')
            self.w(html)
            self.w(u'</div>')
        self.w(u'</div>')

    def entity_call(self, entity):
        self.intro()
        self.comparison_form()
        self.display_diff()



# -*- coding: utf-8 -*-
# copyright 2013 LOGILAB S.A. (Paris, FRANCE), all rights reserved.
# contact http://www.logilab.fr -- mailto:contact@logilab.fr
#
# This program is free software: you can redistribute it and/or modify it under
# the terms of the GNU Lesser General Public License as published by the Free
# Software Foundation, either version 2.1 of the License, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
# details.
#
# You should have received a copy of the GNU Lesser General Public License along
# with this program. If not, see <http://www.gnu.org/licenses/>.

"""cubicweb-vcwiki restructured text rendering tests"""


from utils import VCWikiTC


class VCWikiViewTC(VCWikiTC):

    def assertEntitled(self, expected_title, html):
        self.assertEqual([u'vcwiki - %s' % expected_title],
                         html.find_tag('title'))

    def test_existing_wiki_page(self):
        html = self.wiki_view('subject2/content2')
        self.assertEntitled('content2', html)
        self.assertTrue('content of subject2/content2.rst' in html)

    def test_existing_wiki_page_with_revision(self):
        with self.admin_access.web_request() as req:
            vcwiki = req.entity_from_eid(self.vcwikieid)
            # Check test prerequisite
            vc0 = vcwiki.content('hello.rst', revision=0)
            vc1 = vcwiki.content('hello.rst') # last revision
            new_words = 'modified since its creation'
            self.assertNotIn(new_words, vc0.decode('ascii'))
            self.assertIn(new_words, vc1.decode('ascii'))
            # Check new added content is not present in the view of the old revision
            html = self.wiki_view('hello', rev='0')
            self.assertNotIn(new_words, html)

    def test_non_existing_wiki_page(self, link=True, login=u'admin'):
        html = self.wiki_view('does_not_exist', login=login)
        self.assertEntitled('New wiki page', html)
        self.assertIn('This wiki page does not exist.', html)
        check = getattr(self, 'assert' + str(link))
        check(html.has_link('Create this wiki page?'))

    def test_non_existing_wiki_page_user_cannot_edit(self):
        with self.admin_access.cnx() as cnx:
            self.create_user(cnx, u'spammer')
            cnx.commit()
        self.test_non_existing_wiki_page(link=False, login=u'spammer')


class WikiReSTDirectiveTC(VCWikiTC):

    base_url = u'http://testing.fr/cubicweb/wiki/vcwiki'

    def assertHasLink(self, text, html, url_path, klass):
        attrs = {'class': klass, 'href': self.base_url + url_path}
        links = tuple(html.matching_nodes('a', **attrs))
        self.assertEqual(1, len(links))
        self.assertEqual(text, links[0].text)

    def test_links(self):
        html = self.wiki_view('subject1/with_links')
        self.assertHasLink('simple relative link to content1',
                           html,
                           u'/subject1/content1', u'reference')
        self.assertHasLink('relative link to content2',
                           html,
                           u'/subject2/content2', u'reference')
        self.assertHasLink('absolute link to with_links.rst',
                           html,
                           u'/subject1/with_links', u'reference')
        self.assertHasLink('link to non-existing page',
                           html,
                           u'/subject1/does_not_exist',
                           u'doesnotexist reference')





if __name__ == '__main__':
    from logilab.common.testlib import unittest_main
    unittest_main()
